//----------------------------------------------------------------------------------------
// Board library : esp32 by Espressif 2.0.17
// Board Select  : XIAO_ESP32S3
// NOTE:Need to remove ArduinoBLE(1.3.7) to avoid conflicts
//----------------------------------------------------------------------------------------
// Comparison of current consumption when Advertising MSD 
// MSD 8 bytes data
// TxPower 3dBm
// Advertise 10 times during 1 second and LightSleep for 59 seconds

// 2025/01/16

#include <BLEDevice.h>
#include <BLEServer.h>

// advertising data
#define dataNum 8
uint8_t manufactData[] = { 0xFF, 0xFF,                            /* Manufacture ID */
                           0x02, 0x03, 0x04, 0x05, 0x06, 0x07 };  /* Sample data */

void startAdv(void)
{ 
  // Initialize the BLE Device
  BLEDevice::init("XIAO_ESP32S3");
  // Create BLE Server
  BLEServer *pServer = BLEDevice::createServer();
  // Get Advertising object
  BLEAdvertising *pAdvertising = pServer->getAdvertising();

  // function to set Advertising data
  setAdvData(pAdvertising);

  // Start advertising
  pAdvertising->setScanResponse(false);
  pAdvertising->setAdvertisementType(ADV_TYPE_NONCONN_IND);
  pAdvertising->setMaxInterval(150);  // 100mS
  pAdvertising->setMinInterval(150);
  // N15, N12, N9, N6, N3, N0, P3, P6, P9, P12, P15, P18, P21
  esp_ble_tx_power_set(ESP_BLE_PWR_TYPE_DEFAULT, ESP_PWR_LVL_P3);

  pAdvertising->start();
}

//**************************************************************************************
void setup() {
//  Serial.begin(115200); 
//  while(!Serial);
//  delay(2000);

  esp_sleep_enable_timer_wakeup(59 * 1000000ULL);
}

//********************************************************************************
void loop() {
  startAdv();
  delay(1000);

//  Serial.println("go to Sleep"); Serial.flush();
  esp_deep_sleep_start();
}

// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void setAdvData(BLEAdvertising* pAdvertising)
{
  std::string strData = ""; // for C3, S3
//  String strData = "";  // for C6 
  for(int i = 0; i < dataNum; i++) {
    strData += (char)manufactData[i];
  }
  strData = (char)0xFF + strData;               // add MSD flag
  strData = (char)strData.length() + strData;   // add data length

  for(int i = 0; i < strData.length(); i++) {
//    Serial.printf("%02X ", (uint8_t)strData[i]);
  }
//  Serial.println();

  BLEAdvertisementData oAdvertisementData = BLEAdvertisementData();
  oAdvertisementData.setName("XIAO_ESP32C3");
  oAdvertisementData.setFlags(0x06);    // LE General Discoverable Mode | BR/EDR Not Supported
  oAdvertisementData.addData(strData);  // String strData
  pAdvertising->setAdvertisementData(oAdvertisementData);
}
