//----------------------------------------------------------------------------------------------
// BoardBoard Library : Silicon Labs 2.2.0
// Board Select       : Seeed Studio XIAO MG24 (Sense)
// Plotocol stack     : BLE(Silabs) 
//----------------------------------------------------------------------------------------------
// IMPORTANT NOTE:Cannot upload new sketches during DeepSleep
// When PC0(D0)(0)=LOW and reset, enable to upload. After uploading, be sure to set PC0=HIGH and Reset. 

// Comparison of current consumption when Advertising MSD 
// MSD 8 bytes data
// TxPower 4dBm, on-board Flash off
// Advertise 10 times during 1 second and Light or Deep Sleep for 59 seconds

// 2025/01/16

#include "ArduinoLowPower.h"

// on board Flush SPI_1 pins
#define CS1      PA6   // (21)
#define CLK1     PA0   // (17), D17   
#define MOSI1    PB0   // (15), D15
#define MISO1    PB1   // (16), D16

// on board peripherals pins
#define IMU_EN   PD5   // (19)
#define MIC_EN   PC8   // (22)
#define VBAT_EN  PD3   // (25)
#define RFSW_EN  PB5   // (27)
#define RFSW_CTL PB4   // (26)

// Flash commands
#define READ_DATA     0x03
#define WRITE_ENABLE  0x06
#define PAGE_PROGRAM  0x02
#define SECTOR_ERASE  0x20

// Flash functions
void sendSPI(byte data) {
  for (int i = 0; i < 8; i++) {
    digitalWrite(MOSI1, data & 0x80);
    data <<= 1;
    digitalWrite(CLK1, HIGH);
    delayMicroseconds(1);
    digitalWrite(CLK1, LOW);
    delayMicroseconds(1);
  }
}

void writeEnable() {
  digitalWrite(CS1, LOW);
  sendSPI(WRITE_ENABLE);
  digitalWrite(CS1, HIGH);
}

// advertising data
uint8_t manufactData[] = { 0x02, 0x01, 0x06                    /* Advertising flag */
                         , 0x0B, 0xFF                          /* Manufacture Specific Data flag */
                         , 0xFF, 0xFF                          /* Manufacture ID */
                         , 0x02, 0x03, 0x04, 0x05, 0x06, 0x07  /* Sample data */
                         };

static void ble_start_advertising() {
  static uint8_t advertising_set_handle = 0xff;
  static bool init = true;
  sl_status_t sc;

  if (init) {
    // Create an advertising set
    sc = sl_bt_advertiser_create_set(&advertising_set_handle);
    app_assert_status(sc);

    // Set advertising interval to 100ms, duration 1sec
    sc = sl_bt_advertiser_set_timing(
      advertising_set_handle,
      160,   // minimum advertisement interval (milliseconds * 1.6)
      160,   // maximum advertisement interval (milliseconds * 1.6)
      100,   // advertisement duration (milliseconds * 10)
      0);    // maximum number of advertisement events
    app_assert_status(sc);

    init = false;
  }

  // Set data for advertising
  sc = sl_bt_legacy_advertiser_set_data(advertising_set_handle,
                                        sl_bt_advertiser_advertising_data_packet,
                                        sizeof(manufactData),
                                        (uint8_t*)(&manufactData));
  app_assert_status(sc);

  // Set Tx power
  int16_t *set_power;
  sl_bt_advertiser_set_tx_power(advertising_set_handle, 40, set_power); // 4dBm

  // Start advertising and enable connections
  sc = sl_bt_legacy_advertiser_start(advertising_set_handle, sl_bt_advertiser_connectable_scannable);
  app_assert_status(sc);
}

// **********************************************************************************************************
void setup()
{
  // builtin LED
  pinMode(LED_BUILTIN, OUTPUT);
  digitalWrite(LED_BUILTIN, HIGH);

  // on board flash
  pinMode(CS1, OUTPUT);
  pinMode(CLK1, OUTPUT);
  pinMode(MOSI1, OUTPUT);
  pinMode(MISO1, INPUT);
  digitalWrite(CS1, HIGH);   // CS1 HIGH

  // on board peripherals
  pinMode(IMU_EN, OUTPUT);
  pinMode(MIC_EN, OUTPUT);
  pinMode(VBAT_EN, OUTPUT);
  pinMode(RFSW_EN, OUTPUT);
  pinMode(RFSW_CTL, OUTPUT);
  digitalWrite(IMU_EN, LOW);   // IMU Power OFF
  digitalWrite(MIC_EN, LOW);   // MIC Power OFF
  digitalWrite(VBAT_EN, LOW);  // VBAT Power OFF
  digitalWrite(RFSW_EN, HIGH); // RFSW Power ON
  digitalWrite(RFSW_CTL, LOW); // on-board antenna

  // Flash Deep Power Down 
  writeEnable();
  digitalWrite(CS1, LOW);
  sendSPI(0xB9);
  digitalWrite(CS1, HIGH);
}

void loop()
{
  ble_start_advertising();
  delay(1000);

  // got to Light Sleep, Deep Sleep
//  LowPower.sleep(59000);
  LowPower.deepSleep(59000);
}


